import gapt.expr.stringInterpolationForExpressions
import gapt.provers.sat.Sat4j
import gapt.provers.verit.VeriT
import gapt.utils.runProcess
import gapt.utils.withTimeout

import scala.concurrent.duration.DurationInt

object Workspace extends App {

  val sequent =
    hols"""
           A <-> B,
           A -> f(A) = f(⊤),
           -A -> f(A) = f(⊥),
           B -> f(B) = f(⊤),
           -B -> f(B) = f(⊥),
           f(A) = f(⊤) & f(B) = f(⊤) -> f(A) = f(B),
           f(A) = f(⊥) & f(B) = f(⊥) -> f(A) = f(B)
          :- #c(f: o > i)(A) = f(B)"""

  println( Sat4j.isValid( sequent ) )

  val expansionProof = VeriT.getExpansionProof( hof"g(#c(f: o > i)(#c(A: o) & #c(B: o))) = g(f(B & A))" ).get

  //  println( expansionProof )

  val extendedSequent =
    hof"(A & B) -> #c(ite: o > i > i > i)(A & B, f(⊤), f(⊥)) = f(⊤)" +:
      hof"-(A & B) -> #c(ite: o > i > i > i)(A & B, f(⊤), f(⊥)) = f(⊥)" +:
      hof"(B & A) -> #c(ite: o > i > i > i)(B & A, f(⊤), f(⊥)) = f(⊤)" +:
      hof"-(B & A) -> #c(ite: o > i > i > i)(B & A, f(⊤), f(⊥)) = f(⊥)" +:
      hof"#c(ite: o > i > i > i)(A & B, f(⊤), f(⊥)) = f(⊤) <-> f(⊤) = #c(ite: o > i > i > i)(A & B, f(⊤), f(⊥))" +:
      hof"#c(ite: o > i > i > i)(A & B, f(⊤), f(⊥)) = f(⊥) <-> f(⊥) = #c(ite: o > i > i > i)(A & B, f(⊤), f(⊥))" +:
      hof"#c(ite: o > i > i > i)(B & A, f(⊤), f(⊥)) = f(⊤) <-> f(⊤) = #c(ite: o > i > i > i)(B & A, f(⊤), f(⊥))" +:
      hof"#c(ite: o > i > i > i)(B & A, f(⊤), f(⊥)) = f(⊥) <-> f(⊥) = #c(ite: o > i > i > i)(B & A, f(⊤), f(⊥))" +:
      hof"f(A & B) = #c(ite: o > i > i > i)(A & B, f(⊤), f(⊥))" +:
      hof"f(B & A) = #c(ite: o > i > i > i)(B & A, f(⊤), f(⊥))" +:
      expansionProof.deep

  println( extendedSequent )

  println( Sat4j.isValid( extendedSequent ) )
}

object ExternalProcessWithTimeout extends App {
  withTimeout( 10 seconds ) {
    runProcess(
      Seq( "veriT", "--proof=-", "--disable-print-success", "--disable-banner", "/home/jannik/Documents/SMT-LIB-benchmarks/QF_UF/eq_diamond/eq_diamond20.smt2" ) )
  }
}
